////////////////////////////////////////////////////
// kello_hw4_v06.c
////////////////////////////////////////////////////
// Copyright 2017 Jaakko Kairus
// jaakko@kair.us
//
// You may freely use and modify this software for personal use.
// Commercial use of this software or parts of it is prohibited
// without written permission from author.
// DISCLAIMER: This software comes with ABSOLUTELY NO WARRANTY
//
// For more information about this project see project web page at
// http://kair.us/projects/clock/v4/index.html
//
// Compiled with CCS compiler version 5.054
// Total compiled length 8503 bytes
///////////////////////////////////////////////////
// Version history
// v0.1 13.2.2017
//      Initial version. Support for TLC5925 and PCF2129. Length 1260 bytes.
// v0.2 30.6.2017
//		Added initial support for remote (NEC coding). Length 1771 bytes.
// v0.3 6.9.2017
//		Moved remote decoding stuff to separate file (ir_machine.c) and changed remote
//		data type to a struct compatible with irmp (https://www.mikrocontroller.net/articles/IRMP)
//		Not however using IRMP to avoid getting involved with all the GNU GPL stuff.
//		Added support for showing remote control codes and learn mode to teach clock
//		to use any NEC protocol remote. Learned codes are stored to PIC EEPROM. Added
//		support to set date and time. Added support to adjust brightness. Brightness
//		and display mode are stored to PCF2129 NVRAM (using alarm registers).
//		Added temperature readout for two 10k NTC sensors. Fixed updating the date
//		when day changes. Length 7145 bytes.
// v0.4 8.10.2017
//		Changed ir_machine polling interval from 100s to 64s. This allows using any
//		of the timers to generate the required polling cycle. Changed timer interrupts
//		from timer 2 to timer 0 and 64 s. Frees timer 2 for driving buzzer.
//		Added hour beep function with possibility to set to short, long or off.
//		Better joystick operation with debounced press and release. Length 7457 bytes.
// v0.5 6.12.2017
//		Added support for RC5, Samsung and JVC IR protocols. Protocol is recognized 
//		automatically by the ir_machine() function. The identify menu now also shows
//		these codings. Length 7634 bytes.
// v0.6 23.12.2017
//		Added support for SIRC and NEC42 IR protocols. Rewrote RC5 decoder to sync
//		on every bit. Works now more reliably with different RC5 remotes. Added
//		hrbeep saving. Fixed displaying negative temperature values. Changed brightness
//		to show milliamperes instead of DAC values. Length 8503 bytes.


#define FW_VERSION "v0.6 "
#define OPAMP_NI_TO_DAC1 0x04	// Needed because error in CCS include file

#define RTC_CLK PIN_A6
#define TLC5925_LE PIN_C0

#define IRin PIN_A5

#define REXT 1000	// TLC5925 current set resistor value in ohms

#include <16F1788.h>
#device ADC=12;
#DEVICE PASS_STRINGS=IN_RAM // allows passing constant char arrays to tlc5925.c function
							// copies string from rom to ram when needed

#include <stdlib.h>
#include <stdio.h>
#include <internal_eeprom.c>	// for reading / writing int16
//#include "shiit.h"
#fuses INTRC_IO,NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV25,NOPROTECT,NOLVP,NODEBUG,PUT,NOMCLR,NOCLKOUT,NOVCAP
#use delay(int=16MHz, clock=32MHz)
#use FAST_IO(a)
#use FAST_IO(b)
#use FAST_IO(c)
#use FAST_IO(e)
#use spi(FORCE_HW,BAUD=1000000, BITS=8, SAMPLE_RISE)
#use i2c(master,sda=PIN_B3,scl=PIN_B0,fast=100000)
#use pwm(CCP1, OUTPUT=PIN_C2, TIMER=2, PWM_OFF)
#include "pcf2129.c"
#include "tlc5925.c"
#include "ir_machine.c"
#include "ntc_temperature.c"

//unsigned int8 ROM envelope_tablee[10] = {0,20,40,60,80,100,120,140,160,180};

int8 seconds=0;
int8 minutes=0;
int8 hours=0;
int8 day=0;
int8 month=0;
int8 year=0;
int8 dow=0;

int8 brightness=130;
int8 hrbeep=0;
int16 buzzerDuration=0;

unsigned int16 repeatcounter=0;

unsigned int8 joyshadow, joytimer,releasetimer;

int1 ir_received = FALSE;

int8 pending=0;
#bit PENDING_RTC_SAVE = pending.0
#bit PENDING_RTC_READ = pending.1
#bit PENDING_STATE_SAVE = pending.2
#bit PENDING_BRIGHTNESS_SAVE = pending.3
#bit PENDING_REMOTE_SAVE = pending.4
#bit PENDING_HRBEEP_SAVE = pending.5
#bit PENDING_BRIGHTNESS_SET = pending.6

typedef enum {
        ST_TIME,
        ST_MENU_TIME,
        ST_DATE,
        ST_MENU_DATE,
        ST_IN_TEMPERATURE,
        ST_MENU_IN_TEMPERATURE,
        ST_OUT_TEMPERATURE,
        ST_MENU_OUT_TEMPERATURE,
        ST_TOGGLE,
        ST_MENU_TOGGLE,
        ST_MENU_SET,
        ST_MENU_SET_TIME,
        ST_SET_HOURS,
        ST_SET_MINUTES,
        ST_SET_SECONDS,
        ST_MENU_SET_DATE,
        ST_SET_DAY,
        ST_SET_MONTH,
        ST_SET_YEAR,
        ST_MENU_SET_BRIGHTNESS,
        ST_SET_BRIGHTNESS,
        ST_MENU_SET_HRBEEP,
        ST_SET_HRBEEP,
        ST_MENU_SET_REMOTE,
        ST_MENU_REMOTE_IDENTIFY,
        ST_REMOTE_IDENTIFY,
        ST_MENU_REMOTE_SHOW_CODE,
        ST_REMOTE_SHOW_CODE,
        ST_MENU_LEARN_REMOTE,
        ST_LEARN_REMOTE_UP,
        ST_LEARN_REMOTE_DOWN,
        ST_LEARN_REMOTE_LEFT,
        ST_LEARN_REMOTE_RIGHT,
        ST_LEARN_REMOTE_BUTTON,
        ST_MENU_REMOTE_RETURN,
        ST_MENU_SET_VERSION,
        ST_VERSION,
        ST_MENU_SET_RETURN,
        ST_UNDEFINED
} MENU_STATES;
MENU_STATES menu_state = ST_TIME;

typedef enum {
        EVT_UP,
        EVT_DOWN,
        EVT_LEFT,
        EVT_RIGHT,
        EVT_BUTTON,
        EVT_LONG_PRESS,
        EVT_NONE
} EVENTS;
EVENTS event = EVT_NONE;

void read_jstk(void);
void code_to_button(void);
void menuMachine();
void displayMachine();
void adjust_with_limits(int8 &adjust, int8 low_limit, int8 high_limit);
void save_remote();
void load_remote();
void handle_pending_tasks();
void handleBuzzer();


IR_PACKET receivedPacket;

int8 remoteProtocol;
int16 remoteAddress;
int16 remoteCommandUp;
int16 remoteCommandDown;
int16 remoteCommandLeft;
int16 remoteCommandRight;
int16 remoteCommandButton;

int8 decode_allowed=1;

#int_ioc
void rtcclkisr(void) {
	if (seconds<59)
		seconds++;
	else {
		seconds=0;
		if (minutes<59)
			minutes++;
		else {
			minutes=0;
			if (hrbeep)
				buzzerDuration=7812;
			if (hours<23)
				hours++;
			else {
				hours=0;
				PENDING_RTC_READ=1;
			}
		}	
	}
	if (seconds>54 && minutes==59 && hrbeep==2)	// beep mode = long
		buzzerDuration=1562;	
//	int8 dummy = input_a();
//	clear_interrupt(INT_IOC_A6_H2L);
}	

#int_timer0
void timer0isr(void) {
	output_high(PIN_C7);	// for measuring interrupt length. Currently min 12 s when showing time. Typically max 20 s when decoding and comparing remote codes. Abs. max 30 s.
	ir_machine();
//	receive_rc5();
	ir_received = getIrPacket(&receivedPacket);

	if (ir_received && decode_allowed)
		code_to_button();

//	irmp_ISR();
	read_ntc_sensors();

	read_jstk();
	menuMachine();
	if (buzzerDuration)
		buzzerDuration--;
	output_low(PIN_C7);
}	


void main()
{
	unsigned int16 i=0;
//	unsigned int16 sample;
	
	setup_timer_0(T0_DIV_2); // set timer0 to 64 s (prescaler 2)
	enable_interrupts(INT_TIMER0);
	enable_interrupts(INT_IOC_A6_H2L);	// interrupt for 1 Hz clock signal from RTC chip
	enable_interrupts(GLOBAL);
	set_tris_a(0b01111111);		// all input except ired
	set_tris_b(0b11111111);		// tri-state all port b pins, scl and sda will be controlled by #use_i2c code
	set_tris_c(0b01010000);		// SDI and RX are inputs
	set_tris_e(0x11111000);		
	port_a_pullups(0b01111111);	// pullups for joystick, ir receiver and rtc_clk. not for ired
	port_b_pullups(0b11000000); // pullups for pgc and pgd lines
	port_c_pullups(0b01000000); // pullup for rx input incase no gps module installed
	port_e_pullups(0b00001000); // mclr pullup
		
	setup_adc_ports( sAN11|sAN13,VSS_VDD);	// Temperature sensors
	setup_vref(VREF_ON | VREF_ADC_2v048 | VREF_COMP_DAC_2v048);
	setup_opamp2(OPAMP_ENABLED | OPAMP_HIGH_GBWP_MODE | OPAMP_NI_TO_DAC1);
	setup_dac(DAC_VSS_FVR);

	setup_adc(ADC_CLOCK_DIV_32);	// Gives 1s conversion clock cycle time (fastest recommended)
	set_adc_channel(11);
	sums_ready=0;	// enables ADC conversions

	output_low(PIN_C2);	// ensure the off state of buzzer doesn't present a DC voltage over the buzzer

//	setup_timer_2(T2_DIV_BY_64,31,16); // 4 kHz
//	set_pwm1_duty(15);
//	setup_timer_2(T2_DIV_BY_64,52,16); // 2.4 kHz
//	set_pwm1_duty(26);
	setup_timer_2(T2_DIV_BY_64,125,1); // 1 kHz (8 MHz / 64 / 125)
	set_pwm1_duty(62);
	delay_ms(250);
	setup_ccp1(CCP_OFF);
	output_low(PIN_C1);	// /OE low = display on
	
	if (pcf2129_init()) {
		pcf2129_set_byte(menu_state, PCF2129_SEC_ALARM_REG);	// Clock has been stopped because no battery installed
		pcf2129_set_byte(brightness, PCF2129_MIN_ALARM_REG);	// or first time powered
		pcf2129_set_byte(hrbeep, PCF2129_HOUR_ALARM_REG);
	}
	else {
		pcf2129_get_byte(menu_state, PCF2129_SEC_ALARM_REG);	// Alaram registers are used to store variables
		pcf2129_get_byte(brightness, PCF2129_MIN_ALARM_REG);	// which change often
		pcf2129_get_byte(hrbeep, PCF2129_HOUR_ALARM_REG);
	}	

//	if (menu_state >= ST_UNDEFINED)		// Some sanity checks for values stored to SRAM
//		menu_state = ST_TIME;
//	if (brightness > 149)
//		brightness = 130;

	
	load_remote();

	PENDING_RTC_READ = 1;
	PENDING_BRIGHTNESS_SET = 1;
	
	/* Infinite loop */
	while(1)
	{
		if (pending)
			handle_pending_tasks();

		displayMachine();


		//tlc5925_show_string("v1.00");
		// tlc5925_show_scroll("kaks mummoo meni mustikkaan mut toinen ei mahtunu", 150, &new_press);
		//tlc5925_show_time(hours, minutes, seconds);
		//delay_ms(1000);
		//tlc5925_show_scroll("kauas on pitka matka", 150, &new_press);
		//tlc5925_show_time(hours, minutes, seconds);
		//delay_ms(1000);
		//tlc5925_show_scroll("jarru on vauhdin surma", 150, &new_press);
		//tlc5925_show_time(hours, minutes, seconds);
		//delay_ms(1000);
		//tlc5925_show_scroll("sa oot siella ylhaalla ihan up yours", 150, &new_press);
		//tlc5925_show_time(hours, minutes, seconds);
		//delay_ms(1000);
		//tlc5925_show_scroll("joopa joo sanoivat rengit paissaan", 150, &new_press);


		//read_program_memory(0x1000+i, &sample, 1);
		//dac_write(0xff & sample);
		//i++;
		//if (i==8822)
		//	i=0;
		
		//dac_write(i&0xff);
		i++;
		if (i==10)
			i=0;
		
		handleBuzzer();

		if (menu_state == ST_IN_TEMPERATURE || menu_state == ST_OUT_TEMPERATURE || menu_state == ST_TOGGLE) {
			lookup_32bit(&in_sum, &in_temp, &inhyst);
			lookup_32bit(&out_sum, &out_temp, &outhyst);
			sums_ready=0;
		}	

	}	// end while
}

void menuMachine()
{
	switch(menu_state)
	{
		case ST_TIME:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_TIME;
					break;
			}
			break;
		case ST_MENU_TIME:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_TIME;
					PENDING_STATE_SAVE=1;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_DATE;
					break;
			}
			break;	
		case ST_DATE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_DATE;
					break;
			}
			break;	
		case ST_MENU_DATE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_DATE;
					PENDING_STATE_SAVE=1;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_TIME;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_IN_TEMPERATURE;
					break;
			}
			break;	
		case ST_IN_TEMPERATURE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_IN_TEMPERATURE;
					break;
			}
			break;	
		case ST_MENU_IN_TEMPERATURE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_IN_TEMPERATURE;
					PENDING_STATE_SAVE=1;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_DATE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_OUT_TEMPERATURE;
					break;
			}
			break;	
		case ST_OUT_TEMPERATURE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_OUT_TEMPERATURE;
					break;
			}
			break;	
		case ST_MENU_OUT_TEMPERATURE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_OUT_TEMPERATURE;
					PENDING_STATE_SAVE=1;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_IN_TEMPERATURE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_TOGGLE;
					break;
			}
			break;
		case ST_TOGGLE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_TOGGLE;
					break;
			}
			break;	
		case ST_MENU_TOGGLE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_TOGGLE;
					PENDING_STATE_SAVE=1;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_OUT_TEMPERATURE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET;
					break;
			}
			break;
		case ST_MENU_SET:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_TIME;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_TOGGLE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_TIME;
					break;
			}
			break;
		case ST_MENU_SET_TIME:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_SET_HOURS;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_RETURN;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_DATE;
					break;
			}
			break;
		case ST_SET_HOURS:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_TIME;
					PENDING_RTC_SAVE = 1;
					break;
				case EVT_LEFT:
					menu_state = ST_SET_SECONDS;
					break;
				case EVT_RIGHT:
					menu_state = ST_SET_MINUTES;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(hours, 0, 23);
					break;
			}
			break;
		case ST_SET_MINUTES:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_TIME;
					PENDING_RTC_SAVE = 1;
					break;
				case EVT_LEFT:
					menu_state = ST_SET_HOURS;
					break;
				case EVT_RIGHT:
					menu_state = ST_SET_SECONDS;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(minutes, 0, 59);
					break;
			}
			break;
		case ST_SET_SECONDS:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_TIME;
					PENDING_RTC_SAVE = 1;
					break;
				case EVT_LEFT:
					menu_state = ST_SET_MINUTES;
					break;
				case EVT_RIGHT:
					menu_state = ST_SET_HOURS;
					break;
				case EVT_NONE:
					break;
				default:
					if (seconds>29) {
						if (minutes<59)
							minutes++;
						else
							minutes=0;
					}
					seconds=0;	
					break;
			}
			break;
		case ST_MENU_SET_DATE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_SET_DAY;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_TIME;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_BRIGHTNESS;
					break;
			}
			break;
		case ST_SET_DAY:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_DATE;
					PENDING_RTC_SAVE = 1;
					break;
				case EVT_LEFT:
					menu_state = ST_SET_YEAR;
					break;
				case EVT_RIGHT:
					menu_state = ST_SET_MONTH;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(day, 1, 31);
					break;
			}
			break;
		case ST_SET_MONTH:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_DATE;
					PENDING_RTC_SAVE = 1;
					break;
				case EVT_LEFT:
					menu_state = ST_SET_DAY;
					break;
				case EVT_RIGHT:
					menu_state = ST_SET_YEAR;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(month, 1, 12);
					break;
			}
			break;
		case ST_SET_YEAR:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_DATE;
					PENDING_RTC_SAVE = 1;
					break;
				case EVT_LEFT:
					menu_state = ST_SET_MONTH;
					break;
				case EVT_RIGHT:
					menu_state = ST_SET_DAY;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(year, 0, 99);
					break;
			}
			break;
		case ST_MENU_SET_BRIGHTNESS:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_SET_BRIGHTNESS;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_DATE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_HRBEEP;
					break;
			}
			break;
		case ST_SET_BRIGHTNESS:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_BRIGHTNESS;
					PENDING_BRIGHTNESS_SAVE = 1;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(brightness, 1, 30);	// 30 current settings from 0.5 mA to 15 mA
					PENDING_BRIGHTNESS_SET = 1;
//					adjust_with_limits(brightness, 0, 150);	// Rext voltage 1.21V, DAC output max 2.048V -> max DAC value 150
					break;
			}
			break;
		case ST_MENU_SET_HRBEEP:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_SET_HRBEEP;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_BRIGHTNESS;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_REMOTE;
					break;
			}
			break;
		case ST_SET_HRBEEP:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_HRBEEP;
					PENDING_HRBEEP_SAVE = 1;
					break;
				case EVT_NONE:
					break;
				default:
					adjust_with_limits(hrbeep, 0, 2);	// 0=none, 1=short, 2=long
					break;
			}
			break;
		case ST_MENU_SET_REMOTE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_REMOTE_IDENTIFY;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_HRBEEP;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_VERSION;
					break;
			}
			break;
		case ST_MENU_REMOTE_IDENTIFY:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_REMOTE_IDENTIFY;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_REMOTE_RETURN;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_REMOTE_SHOW_CODE;
					break;
			}
			break;
		case ST_REMOTE_IDENTIFY:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_REMOTE_IDENTIFY;
					break;
			}
			break;
		case ST_MENU_REMOTE_SHOW_CODE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_REMOTE_SHOW_CODE;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_REMOTE_IDENTIFY;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
			}
			break;
		case ST_REMOTE_SHOW_CODE:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_REMOTE_SHOW_CODE;
					break;
			}
			break;
		case ST_MENU_LEARN_REMOTE:
			decode_allowed=1;
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_LEARN_REMOTE_UP;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_REMOTE_SHOW_CODE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_REMOTE_RETURN;
					break;
			}
			break;
		case ST_LEARN_REMOTE_UP:
			decode_allowed=0;
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
			}
			if (ir_received) {
				remoteProtocol = receivedPacket.protocol;
				remoteAddress = receivedPacket.address;
				remoteCommandUp = receivedPacket.command;
				ir_received = FALSE;
				menu_state = ST_LEARN_REMOTE_DOWN;
			}	
			break;
		case ST_LEARN_REMOTE_DOWN:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
			}
			if (ir_received) {
				remoteCommandDown = receivedPacket.command;
				ir_received = FALSE;
				menu_state = ST_LEARN_REMOTE_LEFT;
			}	
			break;
		case ST_LEARN_REMOTE_LEFT:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
			}
			if (ir_received) {
				remoteCommandLeft = receivedPacket.command;
				ir_received = FALSE;
				menu_state = ST_LEARN_REMOTE_RIGHT;
			}	
			break;
		case ST_LEARN_REMOTE_RIGHT:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
			}
			if (ir_received) {
				remoteCommandRight = receivedPacket.command;
				ir_received = FALSE;
				menu_state = ST_LEARN_REMOTE_BUTTON;
			}	
			break;
		case ST_LEARN_REMOTE_BUTTON:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
			}
			if (ir_received) {
				remoteCommandButton = receivedPacket.command;
				ir_received = FALSE;
				menu_state = ST_MENU_LEARN_REMOTE;
				PENDING_REMOTE_SAVE = 1;
			}	
			break;
		case ST_MENU_REMOTE_RETURN:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_REMOTE;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_LEARN_REMOTE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_REMOTE_IDENTIFY;
					break;
			}
			break;
		case ST_MENU_SET_VERSION:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_VERSION;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_REMOTE;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_RETURN;
					break;
			}
			break;
		case ST_VERSION:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET_VERSION;
					break;
			}
			break;
		case ST_MENU_SET_RETURN:
			switch(event)
			{
				case EVT_BUTTON:
					menu_state = ST_MENU_SET;
					break;
				case EVT_LEFT:
					menu_state = ST_MENU_SET_VERSION;
					break;
				case EVT_RIGHT:
					menu_state = ST_MENU_SET_TIME;
					break;
			}
			break;


	} // end menu case
	event = EVT_NONE;
}

void displayMachine()
{
	switch(menu_state)
	{
		case ST_TIME:
			tlc5925_show_time(hours, minutes, seconds);
			break;
		case ST_MENU_TIME:
			tlc5925_show_string("hour");
			break;	
		case ST_DATE:
			tlc5925_show_date(day, month, year);
			break;
		case ST_MENU_DATE:
			tlc5925_show_string("date");
			break;	
		case ST_IN_TEMPERATURE:
			tlc5925_show_temperature(in_temp);
			break;
		case ST_MENU_IN_TEMPERATURE:
			tlc5925_show_string("intemp");
			break;	
		case ST_OUT_TEMPERATURE:
			tlc5925_show_temperature(out_temp);
			break;
		case ST_MENU_OUT_TEMPERATURE:
			tlc5925_show_string("outemp");
			break;	
		case ST_TOGGLE:
			if(seconds%10==0 || seconds%10==1)
				tlc5925_show_date(day, month, year);
			else
				tlc5925_show_time(hours, minutes, seconds);
			break;
		case ST_MENU_TOGGLE:
			tlc5925_show_string("toggle");
			break;
		case ST_MENU_SET:
			tlc5925_show_string("setup");
			break;
		case ST_MENU_SET_TIME:
			tlc5925_show_string("sethou");
			break;	
		case ST_SET_HOURS:
			tlc5925_show_time_with_dimmer(hours, minutes, seconds, 1);	
			break;
		case ST_SET_MINUTES:
			tlc5925_show_time_with_dimmer(hours, minutes, seconds, 2);
			break;
		case ST_SET_SECONDS:
			tlc5925_show_time_with_dimmer(hours, minutes, seconds, 3);
			break;
		case ST_MENU_SET_DATE:
			tlc5925_show_string("setdat");
			break;	
		case ST_SET_DAY:
			tlc5925_show_date_with_dimmer(day, month, year, 1);
			break;
		case ST_SET_MONTH:
			tlc5925_show_date_with_dimmer(day, month, year, 2);
			break;
		case ST_SET_YEAR:
			tlc5925_show_date_with_dimmer(day, month, year, 3);
			break;
		case ST_MENU_SET_BRIGHTNESS:
			tlc5925_show_string("bright");
			break;
		case ST_SET_BRIGHTNESS:
			
//			tlc5925_show_number(151 - brightness*REXT/288);
			tlc5925_show_number((brightness<<2)+brightness);	// much less code than *5
			break;
		case ST_MENU_SET_HRBEEP:
			tlc5925_show_string("hrbeep");
			break;
		case ST_SET_HRBEEP:
			if (hrbeep==0)
				tlc5925_show_string("none");
			else if (hrbeep==1)
				tlc5925_show_string("short");
			else
				tlc5925_show_string("long");
			break;
		case ST_MENU_SET_REMOTE:
			tlc5925_show_string("remote");
			break;
		case ST_MENU_REMOTE_IDENTIFY:
			tlc5925_show_string("identi");
			break;
		case ST_REMOTE_IDENTIFY:
			/*
			if (receivedPacket.protocol) {
				repeatcounter++;
				receivedPacket.protocol=0;
			}
			tlc5925_show_number(repeatcounter);
			*/
			
			switch (receivedPacket.protocol)
			{
				case 0:
					tlc5925_show_string("none");
					break;
				case 1:
					if (bits_received==20)
						tlc5925_show_string("sirc20");
					else if (bits_received==15)
						tlc5925_show_string("sirc15");
					else
						tlc5925_show_string("sirc12");
					break;
				case 2:
					tlc5925_show_string("nec");
					break;
				case 3:
					tlc5925_show_string("samsun");
					break;
				case 7:
					tlc5925_show_string("rc-5");
					break;
				case 20:
					tlc5925_show_string("jvc");
					break;
				case 28:
					tlc5925_show_string("nec42");
					break;
			}
			
			break;
		case ST_MENU_REMOTE_SHOW_CODE:
			tlc5925_show_string("shocod");
			break;
		case ST_REMOTE_SHOW_CODE:
			tlc5925_show_hex(receivedPacket.address>>8, receivedPacket.address, receivedPacket.command);
			break;
		case ST_MENU_LEARN_REMOTE:
			tlc5925_show_string("learn");
			break;
		case ST_LEARN_REMOTE_UP:
			tlc5925_show_string("up");
			break;
		case ST_LEARN_REMOTE_DOWN:
			tlc5925_show_string("down");
			break;
		case ST_LEARN_REMOTE_LEFT:
			tlc5925_show_string("left");
			break;
		case ST_LEARN_REMOTE_RIGHT:
			tlc5925_show_string("right");
			break;
		case ST_LEARN_REMOTE_BUTTON:
			tlc5925_show_string("button");
			break;
		case ST_MENU_REMOTE_RETURN:
			tlc5925_show_string("return");
			break;
		case ST_MENU_SET_VERSION:
			tlc5925_show_string("versio");
			break;
		case ST_VERSION:
			tlc5925_show_string(FW_VERSION);
			break;
		case ST_MENU_SET_RETURN:
			tlc5925_show_string("return");
			break;
	} // end display case
}

void read_jstk(void) {
	joyshadow = input_a() & 0x1f;
	
	if (joyshadow!=0x1f || joytimer)	// 15 ms debounce on press
		joytimer++;
	else
		joytimer=0;
	
	if (joyshadow==0x1f && releasetimer)			// 15 ms debounce on release
		releasetimer--;
	else if (joyshadow!=0x1f && joytimer>1)
		releasetimer=255;
	
	if (joytimer == 1 && releasetimer == 0) {
		if (!(joyshadow &0x04))
			event=EVT_DOWN;
		if (!(joyshadow &0x10))
			event=EVT_UP;
		if (!(joyshadow &0x02))
			event=EVT_LEFT;
		if (!(joyshadow &0x08))
			event=EVT_RIGHT;
		if (!(joyshadow &0x01))
			event=EVT_BUTTON;
	}		
}

void code_to_button(void) {
	event = EVT_NONE;
	if (receivedPacket.address == remoteAddress && receivedPacket.protocol == remoteProtocol) {
		if (receivedPacket.command == remoteCommandUp)
			event = EVT_UP;
		else if (receivedPacket.command == remoteCommandDown)
			event = EVT_DOWN;
		else if (receivedPacket.command == remoteCommandButton)
			event = EVT_BUTTON;
		else if (receivedPacket.command == remoteCommandLeft)
			event = EVT_LEFT;
		else if (receivedPacket.command == remoteCommandRight)
			event = EVT_RIGHT;
	}
	ir_received=FALSE;
}

void adjust_with_limits(int8 &adjust, int8 low_limit, int8 high_limit) {
	if (event==EVT_UP) {
		if (adjust<high_limit)
			adjust++;
		else
			adjust=low_limit;
		break;
	}
	else if (event==EVT_DOWN) {	
		if (adjust>low_limit)
			adjust--;
		else
			adjust=high_limit;
		break;
	}
}

void save_remote() {
	write_eeprom(0, remoteProtocol);
	write_int16_eeprom(2, remoteAddress);
	write_int16_eeprom(4, remoteCommandUp);
	write_int16_eeprom(6, remoteCommandDown);
	write_int16_eeprom(8, remoteCommandLeft);
	write_int16_eeprom(10, remoteCommandRight);
	write_int16_eeprom(12, remoteCommandButton);
}

void load_remote() {
	remoteProtocol = read_eeprom(0);
	remoteAddress = read_int16_eeprom(2);
	remoteCommandUp = read_int16_eeprom(4);
	remoteCommandDown = read_int16_eeprom(6);
	remoteCommandLeft = read_int16_eeprom(8);
	remoteCommandRight = read_int16_eeprom(10);
	remoteCommandButton = read_int16_eeprom(12);
}

void handle_pending_tasks() {
	if (PENDING_RTC_READ) {
		pcf2129_get_time_and_date(seconds, minutes, hours, day, dow, month, year);
	}	
	
	if (PENDING_RTC_SAVE) {
		if ((month == 4 || month == 6 || month == 9 || month == 11)&& day > 30)
			day = 30;
		if (year%4 == 0 && month == 2 && day>29)
			day = 29;
		else if (month == 2 && day>28)
			day = 28;
		pcf2129_set_time_and_date(seconds, minutes, hours, day, dow, month, year);
	}	
	
	if (PENDING_STATE_SAVE) {
		pcf2129_set_byte(menu_state, PCF2129_SEC_ALARM_REG);
	}	
	
	if (PENDING_BRIGHTNESS_SAVE) {
		pcf2129_set_byte(brightness, PCF2129_MIN_ALARM_REG);
	}
	
	if (PENDING_HRBEEP_SAVE) {
		pcf2129_set_byte(hrbeep, PCF2129_HOUR_ALARM_REG);
	}
	
	if (PENDING_REMOTE_SAVE) {
		save_remote();
	}

	if (PENDING_BRIGHTNESS_SET) {
		dac_write(151 - brightness*REXT/288);
	}	

	pending=0;
}		

void handleBuzzer() {
	if (buzzerDuration)
		setup_ccp1(CCP_PWM);
	else
		setup_ccp1(CCP_OFF);
}	