////////////////////////////////////////////////////////////////////////////////
///                               TLC5925.C                                  ///
///          Driver for 16-channel constant current LED driver               ///
///                                                                          ///
/// Copyright 2017 Jaakko Kairus                                             ///
/// jaakko@kair.us                                                           ///
///                                                                          ///
/// You may freely use and modify this software for personal use.            ///
/// Commercial use of this software or parts of it is prohibited             ///
/// without written permission from author.                                  ///
/// DISCLAIMER: This software comes with ABSOLUTELY NO WARRANTY              ///
///                                                                          ///
/// For more information about this project see project web page at          ///
/// http://kair.us/projects/clock/v4/index.html                              ///
///                                                                          ///
/// Developed with CCS compiler version 5.054                                ///
///                                                                          ///
////////////////////////////////////////////////////////////////////////////////

#include "7seg_chars_kello_v4.h"
#define DIM_LEVEL 3	// defines how many dimmer function calls have some digits blanked
int8 const seg_table_left[16] =		{_L0,_L1,_L2,_L3,_L4,_L5,_L6,_L7,_L8,_L9,	// 0,1,2,3,4,5,6,7,8,9
									 _LA,_LB,_LC,_LD,_LE,_LF}; 					// A,B,C,D,E,F
int8 const seg_table_right[16] =	{_R0,_R1,_R2,_R3,_R4,_R5,_R6,_R7,_R8,_R9,
									 _RA,_RB,_RC,_RD,_RE,_RF};
int8 const ascii_table_left[26] =	{_LA,_LB,_LC,_LD,_LE,_LF,_LG,_LH,_LI,_LJ,
									 _LK,_LL,_LM,_LN,_LO,_LP,_LQ,_LR,_LS,_LT,
									 _LU,_LV,_LW,_LX,_LY,_LZ};
int8 const ascii_table_right[26] =	{_RA,_RB,_RC,_RD,_RE,_RF,_RG,_RH,_RI,_RJ,
									 _RK,_RL,_RM,_RN,_RO,_RP,_RQ,_RR,_RS,_RT,
									 _RU,_RV,_RW,_RX,_RY,_RZ};
static int8 dimmer;

void tlc5925_show_time(int8 hours, int8 minutes, int8 seconds);
void tlc5925_show_date(int8 day, int8 month, int8 year);
void tlc5925_show_hex(int8 a, int8 b, int8 c);
void tlc5925_show_scroll(char* message,int8 speed,int8* new_press);
void tlc5925_show_string(char* message);
void tlc5925_show_time_with_dimmer(int8 hours, int8 minutes, int8 seconds, int8 dim_mode);
void tlc5925_show_date_with_dimmer(int8 day, int8 month, int8 year, int8 dim_mode);
void tlc5925_show_number(int16 num);
void tlc5925_show_temperature(signed int16 num);


void tlc5925_show_time(int8 hours, int8 minutes, int8 seconds) {
	spi_xfer(seg_table_right[seconds %10]);
	spi_xfer(seg_table_left[(seconds/10)%10] | 0x08);
	spi_xfer(seg_table_right[minutes %10]);
	spi_xfer(seg_table_left[(minutes/10)%10] | 0x08);
	spi_xfer(seg_table_right[hours %10]);
	spi_xfer(seg_table_left[(hours/10)%10]);
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

void tlc5925_show_date(int8 day, int8 month, int8 year) {
	spi_xfer(seg_table_right[year %10]);
	spi_xfer(seg_table_left[(year/10)%10]);
	spi_xfer(seg_table_right[month %10] | 0x04);
	spi_xfer(seg_table_left[(month/10)%10]);
	spi_xfer(seg_table_right[day %10] | 0x04);
	spi_xfer(seg_table_left[(day/10)%10]);
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

void tlc5925_show_hex(int8 a, int8 b, int8 c) {
	spi_xfer(seg_table_right[c %16]);
	spi_xfer(seg_table_left[(c/16)%16]);
	spi_xfer(seg_table_right[b %16]);
	spi_xfer(seg_table_left[(b/16)%16]);
	spi_xfer(seg_table_right[a %16]);
	spi_xfer(seg_table_left[(a/16)%16]);
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

//Sends scrolling string to led display via tlc5925
void tlc5925_show_scroll(char* message,int8 speed,int8* new_press)
{
	int8 i,j;
	int8 buffer[7];
	for (i=0;i<strlen(message)+8;i++) {
		if (*new_press != 0)
			return;
		for (j=0;j<6;j++) {
			if (i+j<7||i+j>strlen(message)+6) // blanks before and after message
				buffer[j]=0x20;	// ascii space
			else
				buffer[j] = message[i+j-7];
		}
		buffer[6]=0x00;	// ensure the string is null terminated
		tlc5925_show_string(buffer);	
		delay_ms(speed);
	}
}

//Sends static string to led display via tlc5925
void tlc5925_show_string(char* message)
{
	int right,k,i=strlen(message)-1;
	for (int j=0;j<6;j++) {
		k=i-j;
		right=(j%2==0);
		if (j>i)
			spi_xfer(_SPACE);
		else if (message[k] == 0x20) // space
			spi_xfer(_SPACE);
		else if (message[k] == '.' || message[k] == ',') {  // comma
			if (right)
				spi_xfer(_R_POINT);
			else
				spi_xfer(_L_POINT);
		}
		else if (message[k] == '-') {  // minus
			if (right)
				spi_xfer(_R_MINUS);
			else
				spi_xfer(_L_MINUS);
		}
		else if (message[k] >= '0' && message[k] <= '9') {  // number
			if (right)
				spi_xfer(seg_table_right[message[k]-'0']);
			else
				spi_xfer(seg_table_left[message[k]-'0']);
		}
		else if (right)
			spi_xfer(ascii_table_right[message[k]-'a']);		// (hopefully) letter
		else
			spi_xfer(ascii_table_left[message[k]-'a']);		// (hopefully) letter
	}	
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

void tlc5925_show_time_with_dimmer(int8 hours, int8 minutes, int8 seconds, int8 dim_mode) {
	if (dimmer<DIM_LEVEL) {
		dimmer++;
		if (dim_mode==3) {
			spi_xfer(seg_table_right[seconds %10]);
			spi_xfer(seg_table_left[(seconds/10)%10] | 0x08);
		}
		else {
			spi_xfer(_SPACE);
			spi_xfer(0x08);
		}
		if (dim_mode==2) {
			spi_xfer(seg_table_right[minutes %10]);
			spi_xfer(seg_table_left[(minutes/10)%10] | 0x08);
		}
		else {
			spi_xfer(_SPACE);
			spi_xfer(0x08);
		}
		if (dim_mode==1) {
			spi_xfer(seg_table_right[hours %10]);
			spi_xfer(seg_table_left[(hours/10)%10]);
		}
		else {
			spi_xfer(_SPACE);
			spi_xfer(_SPACE);
		}
	}
	else {
		dimmer=0;
		tlc5925_show_time(hours,minutes,seconds);
	}	
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

void tlc5925_show_date_with_dimmer(int8 day, int8 month, int8 year, int8 dim_mode) {
	if (dimmer<DIM_LEVEL) {
		dimmer++;
		if (dim_mode==3) {
			spi_xfer(seg_table_right[year %10]);
			spi_xfer(seg_table_left[(year/10)%10]);
		}
		else {
			spi_xfer(_SPACE);
			spi_xfer(_SPACE);
		}
		if (dim_mode==2) {
			spi_xfer(seg_table_right[month %10] | 0x04);
			spi_xfer(seg_table_left[(month/10)%10]);
		}
		else {
			spi_xfer(0x04);
			spi_xfer(_SPACE);
		}
		if (dim_mode==1) {
			spi_xfer(seg_table_right[day %10] | 0x04);
			spi_xfer(seg_table_left[(day/10)%10]);
		}
		else {
			spi_xfer(0x04);
			spi_xfer(_SPACE);
		}
	}
	else {
		dimmer=0;
		tlc5925_show_date(day,month,year);
	}	
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

void tlc5925_show_number(int16 num) {
	int8 j;                       // Digit index
	int8 led_data[6];     // Ram array for segment data
	int16 temp16;
	
	// Get led segments for all four digits. Don't need to divide to get units.
	temp16 = num; 
	for (j=0;j<6;j++) {
		if (j != 0)
			temp16 /= 10;
		led_data[j] = seg_table_right[temp16 %10];
		temp16 /= 10;
		j++;
		led_data[j] = seg_table_left[temp16 %10];
    }

   // Do leading zero blanking
   led_data[5] = _SPACE;
   if (num<10000) led_data[4] = _SPACE;
   if (num<1000)  led_data[3] = _SPACE;
   if (num<100)   led_data[2] = _SPACE;
   if (num<10)    led_data[1] = _SPACE;

   // Add decimal point if required
   //led_data[2] |= 0x80;

   // Send led segments

   for (j=0;j<6;j++) {
	spi_xfer(led_data[j]);
   }
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

void tlc5925_show_temperature(signed int16 num) {	// 8141 bytes before modding to show negs. 8244 after.
	int8 j;                       // Digit index
	int8 led_data[6];     // Ram array for segment data
	signed int16 temp16;
	
	// Get led segments for all four digits. Don't need to divide to get units.
	temp16 = labs(num); 
	for (j=1;j<6;j++) {
		led_data[j] = seg_table_left[temp16 %10];
		temp16 /= 10;
		j++;
		led_data[j] = seg_table_right[temp16 %10];
		temp16 /= 10;
	}

   // Do leading zero blanking
	led_data[5] = _SPACE;
	if (num<1000 && num>-100) led_data[4] = _SPACE;
	if (num<100 && num>-10)  led_data[3] = _SPACE;
	if (num<0 && labs(num)>99) led_data[4] = _R_MINUS;
	if (num<0 && labs(num)<100) led_data[3] = _L_MINUS;

	led_data[0] = _RC;
	
	
   // Add decimal point if required
   led_data[2] |= _R_POINT;

   // Send led segments

   for (j=0;j<6;j++) {
	spi_xfer(led_data[j]);
   }
	output_high(TLC5925_LE);
	output_low(TLC5925_LE);
}

